/**  COMMON UNIFIED CLIENT INTERFACE MANAGMENT CODE */

// This file is not intended to be modified when making a new template / modifying
// a template as it is used by ALL templates.  The intent is that there should be 
// a function call that be overridden in the <templatename>.js file.
//
// The Client class provides some methods that are intended to be overridden by 
// the template's javascript file. These are the Cleint.template* methods, and
// an example on how to override them is provided at the bottom of the Client 
// class (where those method are defined).

var Client = {
	chatEnabled: false,
	firstFocus: true,
	typing: false,
	langID: 10,
	blockedTime: g_WaitTime, // Set this to the minumum number of seconds between logins.
	allowSound: g_AllowSound,
	inFlyout: false,

	lastPushUrl: "",
	lastPopUrl: "",

	is_agent_photo_available: false,

	surveyUrl: "",
	bExitUrl: g_exiturlenabled,
	bSurveyNoExitURL: false,
	exitUrl: null,
	agentsTypingNames: new Array(),
	httpport: g_httpport,
	serverurl: g_serverurl,
	secure: g_secure,
	langurl: g_langurl,
	
	copybtn: null,
	copyEleContent: null,
	textArea: null,
	
	use_ie_clipboard: false,
	clip: null,
	
	curfontsize: 1,
	fontsizes: ["smallfont","normalfont","largefont","xlargefont"],
	fontsizemap: [75,100,125,150],
	
	templateName: "unknown",
	
	supportedFeatures: 			0,
	FEATURE_CHAT: 				1,
	FEATURE_BRANCHSCRIPT: 		2,
	FEATURE_COBROWSE:			4,
	FEATURE_EMAILTRANSCRIPT: 	8,
	FEATURE_UPLOAD:				16,
	FEATURE_ADS:				32,

	SPLITTER_MARGIN: 5,

	MSG_FORMAT_SYSTEM:			0,
	MSG_FORMAT_AGENT:			1,
	MSG_FORMAT_CUST:			2,
	MSG_FORMAT_NOTICE:			3,
	MSG_FORMAT_SCRIPT:			4,
	MSG_FORMAT_TYPING:			6, // 5 is used internally.
	MSG_FORMAT_SYSTEM_TRANS:	128,
	MSG_FORMAT_AGENT_TRANS: 	129,
	MSG_FORMAT_CUST_TRANS:		130,
	MSG_FORMAT_NOTICE_TRANS:	131,
	MSG_FORMAT_SCRIPT_TRANS:	132,
	MSG_FORMAT_TYPING_TRANS:	134,
	MSG_FORMAT_UPLOAD_COMPLETE: 256,
	MSG_FORMAT_NOTRANSCRIPT:    32768,
	MSG_FORMAT_NOALERT:	        65536,

	isChrome: function() {
		var uAgent = navigator.userAgent.toLowerCase();
		return ((uAgent.indexOf("chrome") > -1) || (uAgent.indexOf("crios") > -1));
	},

	isFullScreen: (function() {
		return !(document.location.href.indexOf("fullScreen=false") > 0); // true unless fullScreen=false
	})(),

	startup: function(copybutton,copyElementContent,textarea) {
		this.inFlyout = (document.location.href.indexOf('flyout=1') > 0);
		this.surveyUrl = "";
		Client.iosVersion = Util.iOSVersion();
		Client.ieVersion = Util.ieVersion();

		//Resume suspended session when refreshing naclient page that is not in a flyout.
		if (!this.inFlyout) {
			var suspendIdCookie = this.getCookie("SUSPENDID");
			var sessionIdCookie = this.getCookie("SESSIONID");
			if (suspendIdCookie && sessionIdCookie) {
				//Let cimlogin.aspx handles resuming suspended session
				window.location.href = "/netagent/cimlogin.aspx?questid=" + NAParam.questionnaireid + "&portid=" + NAParam.portalid +
					"&defaultStyleId=" + NAParam.portalid;
				return;
			}
		}

		// Class to indicate whether hosted in flyout
		var htmlElem = document.getElementsByTagName("html")[0];
		Client.addClass(htmlElem, (this.inFlyout ? "inFlyout" : "not-inFlyout"));

		// Flag used for css work-around when font-size < 16px
		Client.addClass(htmlElem, (Client.iosVersion.major >= 9) ? "ios9-up" : "");

		// Class to indicate orientation
		Client.addClass(htmlElem, (isPortrait() ? "portrait" : "landscape"));

		// Special behavior when hosted in an iframe
		if (this.inFlyout) {
			// Listen for messages from a parent window (if present)
			Util.addEventHandler(window, "message", function(event) {
				switch (event.data) {
					case "hideChat":
						// placeholder
						break;
					case "endChat":
						Client.exit();
						break;
				}
			});

			// Handles clicks to links within the flyout chat area.
			Util.addEventHandler(document.getElementById("chattext"), "click", function (event) {
				if (event.target.href) {
					clientDriver.hideFlyoutOnNextPage();
				}
			});

			top.postMessage('{"call":"setUrl","url":"' + document.location.href + '"}', '*');
		}

		// Assume device doesn't respond to touchstart events
		Client.hasScreenKeyboard = false;
		Client.addClass(document.getElementsByTagName("html")[0], "no-screenKeyboard");

		// Create touchstart event handler
		var touchstartFlagger = (function(clientObj) {
			// If touchstart event happens, flag it then remove this handler
			var handler = function(e) {
				clientObj.hasScreenKeyboard = true;
				Client.removeClass(document.getElementsByTagName("html")[0], "no-screenKeyboard");
				Client.addClass(document.getElementsByTagName("html")[0], "screenKeyboard");
				Util.preventElementOnClick("moxielogolink");
				Util.removeEventHandler(document, "touchstart", handler);
			};
			return handler;
		})(Client);

		// Listen for touchstart events
		Util.addEventHandler(document, "touchstart", touchstartFlagger);

		var script = document.createElement("script");
		script.src = "/netagent/Libraries/ToastLibrary/src/js/toast.js";
		script.type = "text/javascript";
		var scriptCss = document.createElement("link");
		scriptCss.href = "/netagent/Libraries/ToastLibrary/src/css/toast.css";
		scriptCss.rel = "stylesheet";
		scriptCss.type = "text/css";
		if (document.getElementsByTagName('head')[0]) {
			document.getElementsByTagName('head')[0].appendChild(script);
			document.getElementsByTagName('head')[0].appendChild(scriptCss);
		}

		this.copybtn = copybutton;
		this.copyEleContent = copyElementContent;
		this.textArea = textarea;
	
		this.docOnMouseDown = this.drag.startDrag;
		this.docOnMouseUp = this.drag.stopDrag;
		this.docOnMouseMove = this.drag.dragDiv;
		document.onmousedown=this.docOnMouseDown;
		document.onmouseup=this.docOnMouseUp;
		document.onmousemove=this.docOnMouseMove;
		
		textarea.onkeyup=Client.textOnKeyUp;
		textarea.onkeydown=Client.textOnKeyDown;
		textarea.onfocus=Client.textOnFocus;
		textarea.onpaste=Client.textOnPaste;
		textarea.onpropertychange=Client.textOnPropertyChange;

		if (!this.inFlyout) {
			//DOS CHECK START
			var now = new Date();
			var cookieSet = false;
			var cookies;
			var temp;
		
			cookies = document.cookie.split(';');
			for(var i=0;i < cookies.length;i++) {
				temp = cookies[i].split('=');
				cookieSet |= (temp[0] == "HTMLClientBlocked");
			}
		
			if (!cookieSet) {
				now.setTime(now.getTime() + 1000 * this.blockedTime);
				document.cookie = "HTMLClientBlocked=true;expires=" + now.toUTCString();
			} else {
				if( typeof outputNetworkLayer != 'undefined' && outputNetworkLayer != null){
					outputNetworkLayer.shutDown();
				}
				window.open("/NetAgent/login_too_soon.aspx?timeout="+this.blockedTime,"_top");
				return false;
			}
			//DOS CHECK END
		}

		this.importFontSize();

		initTemplate();

		if(!g_ShowSendButton){
			this.buttonHide("send",true);
		}
		this.buttonDisable("send", true);
		this.buttonHide("copy",true);

		if (document.getElementById("poweredby")){
			document.getElementById("poweredby").title = g_custCompanyname;
		}

		if(!String.prototype.trim) {
			String.prototype.trim = function () {
				return this.replace(/^\s+|\s+$/g,'');
			};
		}

		this.setSplitter();
		if (Client.iosVersion.major >= 8) {
			var betaSafari = (navigator.userAgent.toLowerCase().indexOf("safari") > -1 && Client.iosVersion.major == 8 && Client.iosVersion.minor == 0 && (Client.iosVersion.build == 0 || Client.iosVersion.build == 1));
			if (betaSafari) { //disable file uploads
				g_ShowAttachButton = false;
			}
		}

		var ret = this.templateStartup();

		if(this.inFlyout){
			this.templateInitFlyout();
		}

		return ret;
	},

	loadStrings: function(){
		var i, x;

		for (var x = 0; x < 2; x++) {
			// populate strings of main doc and iframe if present
			var docRoot = window.document;
			if (x == 1) {
				docRoot = window.frames["ifrmUpload"].document ?
					window.frames["ifrmUpload"].document :
					window.frames["ifrmUpload"].contentWindow.document;
			}
			for(i = 0; i < 250; i++) {
				var element = docRoot.getElementsByName("string"+i);
				if(element != undefined && element != null && element.length>0){
					var text = ""+outputNetworkLayer.getLangString(i);
					var count;
					for(count=0;count<element.length;count++){
						if(element[count]!= undefined && element[count] != null)
							element[count].innerHTML = safeHTMLEncode(text);
					}
				}
			}
			//special cases
			document.title = ""+outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_CUSTOMER_CLIENT_TITLE);

				}

		this.templateLoadStrings();
	},

	//called when connected to the server
	appletReady: function(){
		
		Client.templateConnected();
	},

	changeFontSize: function() {
		this.curfontsize = (this.curfontsize + 1) % this.fontsizes.length;
		this.setFontSize();

		// give browser time to recalculate sizes
		setTimeout(this.setSplitter, 500);
	},

	setFontSize: function(){
		// new using CSS version
		var ele = document.getElementById("container");
		if (ele) {
			for(var i=0;i<this.fontsizes.length;i++){
				this.removeClass(ele,this.fontsizes[i]);
			}
			this.addClass(ele,this.fontsizes[this.curfontsize]);
		}
		this.exportFontSize();

		// IE only: chat input-area's text needs to be repainted after resize
		// TODO: likely quirks-mode problem
		var inputArea = document.getElementById("textarea");
		if ((inputArea) && (inputArea.innerText)) {
			inputArea.innerText = inputArea.innerText;
		}
	},

	// Set font size based on value in cookie.
	importFontSize: function(){
		var fontChoice = parseInt(this.getCookie('FONT_CHOICE'));
		fontChoice = isNaN(fontChoice) ? 1 : fontChoice % this.fontsizemap.length;

		this.curfontsize = fontChoice;
		this.setFontSize();
	},
	
	exportFontSize: function(){
		var expires = new Date();
		expires.setTime(expires.getTime() + (1000 * (24 * 60 * 60 * 30))); //one month
		var hostName = window.location.hostname;
		var domainname = hostName;

		// If hostname is not an ip, set cookie using root domain
		if (hostName.match(/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) == null) {
			var hostParts = hostName.split(".");

			// Use last two parts of domain if available, otherwise just use root
			var firstIndex = (hostParts.length > 1) ? -2 : -1;
			domainname = hostParts.slice(firstIndex).join(".");
			domainname = "." + domainname;
		}

		this.setcookie("FONT_CHOICE=" + this.curfontsize, expires.toUTCString(), '/', domainname);
	},

	setcookie: function(namevalue, expires, path, domain){
		var thecookiestring = namevalue + 
			((expires) ? "; expires=" + expires : "" ) + 
			((path) ? "; path=" + path : "" ) + 
			((domain) ? "; domain=" + domain : "");
		document.cookie = thecookiestring;
	},
	
	getCookie: function(name) {
		var nameEQ = name + "=";
		var ca = document.cookie.split(';');
		for(var i=0;i < ca.length;i++) {
			var c = ca[i];
			while (c.charAt(0)==' ') 
				c = c.substring(1,c.length);
			if (c.indexOf(nameEQ) == 0) 
				return c.substring(nameEQ.length,c.length);
		}
		return null;
	},
	
	addClass: function(ele, clName) {
		if (ele && clName) {
			if (ele.classList != null) {
				ele.classList.add(clName);
			} else {
				ele.className += (ele.className ? ' ' : '') + clName;
			}
		}
	},
	
	removeClass: function(ele,clName){
		if(ele.classList!=null){
			ele.classList.remove(clName);
		}else{
			ele.className = ele.className.replace(new RegExp('(\\s|^)'+clName+'(\\s|$)',"g"),' ');
		}
	},
	
	hasClass: function(ele,clName) {
		if(ele.classList!=null){
			return ele.classList.contains(clName);
		}else{
			return ele.className.match(new RegExp('(\\s|^)'+clName+'(\\s|$)'));
		}
	},

	getChatHeight: function(){
		return Math.max( document.documentElement.clientHeight,
			document.body.scrollHeight, document.documentElement.scrollHeight,
			document.body.offsetHeight, document.documentElement.offsetHeight);
	},
	getChatWidth: function(){
		return Math.max( document.documentElement.clientWidth,
			document.body.scrollWidth, document.documentElement.scrollWidth,
			document.body.offsetWidth, document.documentElement.offsetWidth);
	},
	
	exit: function(){
		var objExit = document.getElementById("exit");
		if (objExit != null && objExit.disabled){
			if(Client.bExitUrl){	        
				Client.showExitUrl();
			}
			return;
		}
		Client.templateDisableButton("exit");
		Client.templateDisableButton("send");

		if (Client.cobrowse.is_cobrowse)
			Client.cobrowse.close();

		outputNetworkLayer.sendExitRequest();

		Client.showExitUrl();
		Client.enableChat(false);	
		
		Client.upload.hideUploadDlg();
		Client.ct.timeoutEnd(false); //don't exit the client here cause already done so in previous line.

		top.postMessage('{"call":"setFlyoutTabText","text":"' + safeHTMLEncode(g_Flyout_ChatNow_Text) + '"}', '*');
		
	},


	setStatusMsg: function(html){
		Client.templateSetStatusMsg(safeHTMLEncode(html));
	},

	setStatusMsgId: function(msgid){
		//this exists so that the template can 'veto' the display of certain status message or control how they are displayed.

		var how = Client.templateGetStatusDisplayType(msgid);
		var html = outputNetworkLayer.getLangString(msgid);

		if (how == 10) {
			Client.templateSetStatusMsg(safeHTMLEncode(html));
		} else if (how > -1 ) {
			var name = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_SYSTEM_TABLE_PREFEXT);
			displayMessage(new Date(), how, safeHTMLEncode(html), null, name, null, null, null, null);
		}
	},

	setExitUrl: function(exitUrl){
		this.exitUrl = exitUrl;
	},
	
	showExitUrl: function(){
		var exiturlenabled = g_exiturlenabled;
		var exitlastpush = g_exitlastpush;
		var exittarget = g_exittarget;
		if(clientDriver.portalexitwindowstyle != "3"){   // What does 3 mean?
			try{
				if(clientDriver.portalexitlastpush.length > 0)
					exitlastpush = (clientDriver.portalexitlastpush == "1" ? true : false);

				if(clientDriver.portalexittarget.length > 0)
					exittarget = clientDriver.portalexittarget
					
				if(clientDriver.portalexiturlenabled.length > 0)
					exiturlenabled = (clientDriver.portalexiturlenabled == "1" ? true : false);
			}catch(e){}
			
			if(exitlastpush){
			    exiturl = Client.lastPushUrl;
			}else{
			    exiturl = this.exitUrl;
			}

			if(outputNetworkLayer != null)
				outputNetworkLayer.shutDown();
			if (clientDriver.SurveyOnSelf && exittarget == "_self"){
				this.bSurveyNoExitURL = true;
			}

			if (exiturl && exiturlenabled && this.bSurveyNoExitURL == false) {
			    if (Client.inFlyout)
			        top.postMessage('{"call":"openWindow", "url":"' + exiturl + '", "target":"' + exittarget + '"}', '*');
                else
				    setTimeout('top.window.open("' + exiturl + '", "' + exittarget + '");', 1000);
				this.bExitUrl = true;
			}
		}		
	},
	
	enableChat: function(enable){
	
		Client.enableInputArea(enable);
		Client.chatEnabled=enable;

		Client.templateEnableChat(enable);
	},
	
	enableInputArea: function(enable){
		this.textArea.disabled = !enable;
		
		Client.templateEnableInputArea(enable);
		
		setTimeout(Client.checkForTyping, 2000);
		
		if(enable){
			Client.templateEnableButton("copy");
			Client.templateEnableButton("print");
		}
	},
	
	sendText: function(){
		Client.trunc();
		clientDriver.sendCustMsg(this.textArea); 
		Client.textArea.value = '';
		try{
			Client.textArea.focus();
		}catch(e){}
		
		Client.templateDisableButton("send");
			
		typing = false;
		Client.templateTextSent();
	},

	getCustomerText: function(){
	    return Client.textArea.value;
	},

	setCustomerText: function(text){
	    Client.textArea.value = text;
	},
	
	buttonDisable: function(buttonid,disable){
		if(disable){
			Client.templateDisableButton(buttonid);
		}else{
			Client.templateEnableButton(buttonid);
		}
	},
	
	buttonHide: function(buttonid,hide){
		if(hide){
			Client.templateHideButton(buttonid);
		}else{
			Client.templateShowButton(buttonid);
		}
	},

	/*Text Entry Code */
	checkSendButton: function(event) {
		var txt = this.textArea.value.replace(/^\s+|\s+$/g, "");
		Client.buttonDisable("send", (txt.length == 0));

		var shift;
		var key;
		
		if(event==null) //for ie
			event=window.event;

		if (event != null) {
			shift = event.shiftKey;
			key = event.keyCode;
		
			// If enter is pressed without shift, then submit the text, but do not pass the event through.
			if((key == 13) && (txt.length == 0)){
			   Client.textArea.value = "";
			   Client.textArea.focus();
			}
		}

		return true;
	},

	trunc: function(event){
		var text = this.textArea;
		if(event==null) //for ie
			event=window.event;
		if(text!=null){
			if (text.value == "\n")
				text.value = "";
			if (event != null && event.keyCode != 13) Client.typing = true;

			var len = text.value.length;
			
			Client.templateTextLength(len);
			
			if(! g_MaxMsgSize )
				return true;
			
			if (len > g_MaxMsgSize ){
				text.value = text.value.substr(0, g_MaxMsgSize);
				var toast_container = document.getElementById('android_toast_container');
				if (! toast_container)
					var toast1 = new Android_Toast({content: outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_EDIT_ERROR_MSG), position: 'bottom'});
			}
		}
		return true;
	},

	checkForTyping: function() {
		if (!Client.chatEnabled) {
			return;
		}
		if (Client.typing) {
			outputNetworkLayer.setCustomerTyping(Client.typing);
		}
		setTimeout(Client.checkForTyping, 2000);
		Client.typing = false;
	},

	checkForReturn: function(event) {
		var shift;
		var key;

		if(event==null) //for ie
			event=window.event;

		shift = event.shiftKey;
		key = event.keyCode;

		// If enter is pressed without shift, then submit the text, but do not pass the event through.
		if ((key == 13) && (! shift)) {
			var text = this.textArea;
			if (text != null){
				var txt = text.value.replace(/^\s+|\s+$/g, "");
				if(txt.length != 0){
					Client.sendText();
				}
			}
			return false;
		}
		Client.trunc(event);
		return true;
	},

	//Functions to assign to TextArea
	textOnKeyUp: function(event){
		return (Client.trunc(event) && Client.checkSendButton(event));
	},
	textOnKeyDown: function(event){
		if (Client.checkInputText) {
			// Chrome on iPhone doesn't fire onFocus properly, which we use to clear this text
			// so clear it hear on first keyDown
			if (Client.textArea.value == outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_TEXTAREA_TYPE_HERE))
			Client.textArea.value = "";
			Client.checkInputText = false;
		}
		return Client.checkForReturn(event);
	},
	textOnFocus: function(event){
		if(Client.templateTextOnFocus){
			Client.templateTextOnFocus(event);
		}else if(Client.firstFocus){
			Client.textArea.value="";
		}
		Client.firstFocus=false;
	},
	textOnPaste: function(event){
		setTimeout(function() {Client.checkSendButton(event)}, 100);
	},
	textOnPropertyChange: function(event){
		Client.checkSendButton(event);
	},
	/*end Text Entry Code*/

	/* Code to implement draggable divs */
	drag: {
		dragTarget: null,
		drag: false,
		dragX: 0,
		dragY: 0,
		dragOffX: 0,
		dragOffY: 0,
		frameWidth: 400,
		frameHeight: 400,
		startDrag: function(e){
			// determine event object
			if(!e){
				var e=window.event
			};
			// determine target element
			Client.drag.dragTarget = null;
			var targ=e.target?e.target:e.srcElement;
			var c=0;
			for(c=0;c<3&&Client.drag.dragTarget==null;c++){
				if(targ.id=='bstitle'){
					Client.drag.dragTarget = document.getElementById('branchscript');
				}else if(targ.id=='btitle'){
					Client.drag.dragTarget = document.getElementById('webbrowser');
				}else if(targ.id=='ctimeouttitle'){
					Client.drag.dragTarget = document.getElementById('custtimeout');
				}else if(targ.id=='uploadtitle'){
					Client.drag.dragTarget = document.getElementById('uploadwindow');
				}else if(targ.id=='emailtitle'){
					Client.drag.dragTarget = document.getElementById('emailtranscript');
				}
				targ = targ.parentNode;
				if(targ == null)
					c=3;
			}
			if(c==3 || Client.drag.dragTarget==null){
				return;
			}
			// calculate event X,Y coordinates
			Client.drag.dragOffX=e.clientX;
			Client.drag.dragOffY=e.clientY;
			// assign default values for top and left properties
			if(!Client.drag.dragTarget.style.left || Client.drag.dragTarget.style.left==undefined || Client.drag.dragTarget.style.left==null || Client.drag.dragTarget.style.left=='')
				Client.drag.dragTarget.style.left='100px';
			if(!Client.drag.dragTarget.style.top || Client.drag.dragTarget.style.top==undefined || Client.drag.dragTarget.style.top==null || Client.drag.dragTarget.style.top=='')
				Client.drag.dragTarget.style.top='100px';

			// calculate integer values for top and left properties
			Client.drag.dragX=parseInt(Client.drag.dragTarget.style.left);
			Client.drag.dragY=parseInt(Client.drag.dragTarget.style.top);
			Client.drag.drag=true;
			
			//now get the frame width / height which is needed to limit dragging area
			if (document.documentElement && document.documentElement.clientWidth){
				Client.drag.frameWidth = document.documentElement.clientWidth;
				Client.drag.frameHeight = document.documentElement.clientHeight;
			}else if (document.body){
				Client.drag.frameWidth = document.body.clientWidth;
				Client.drag.frameHeight = document.body.clientHeight;
			}else if (self.innerWidth){
				Client.drag.frameWidth = self.innerWidth;
				Client.drag.frameHeight = self.innerHeight;
			}
			//and subtract the div width
			if(Client.drag.dragTarget.width != undefined && Client.drag.dragTarget.width != null && !(isNaN(parseInt(Client.drag.dragTarget.width))) ){
				Client.drag.frameWidth = Client.drag.frameWidth - parseInt(Client.drag.dragTarget.width);
			}else if(Client.drag.dragTarget.style.width != undefined && Client.drag.dragTarget.style.width != null && !(isNaN(parseInt(Client.drag.dragTarget.style.width))) ){
				Client.drag.frameWidth = Client.drag.frameWidth - parseInt(Client.drag.dragTarget.style.width);
			}if(Client.drag.dragTarget.clientHeight != undefined && Client.drag.dragTarget.clientHeight != null && !(isNaN(parseInt(Client.drag.dragTarget.clientHeight))) ){
				Client.drag.frameWidth = Client.drag.frameWidth - parseInt(Client.drag.dragTarget.clientWidth);
			}else{
				Client.drag.frameWidth = Client.drag.frameWidth - 300;
			}
			if(Client.drag.dragTarget.height != undefined && Client.drag.dragTarget.height != null && !(isNaN(parseInt(Client.drag.dragTarget.height))) ){
				Client.drag.frameHeight = Client.drag.frameHeight - parseInt(Client.drag.dragTarget.height);
			}else if(Client.drag.dragTarget.style.height != undefined && Client.drag.dragTarget.style.height != null && !(isNaN(parseInt(Client.drag.dragTarget.style.height))) ){
				Client.drag.frameHeight = Client.drag.frameHeight - parseInt(Client.drag.dragTarget.style.height);
			}if(Client.drag.dragTarget.clientHeight != undefined && Client.drag.dragTarget.clientHeight != null && !(isNaN(parseInt(Client.drag.dragTarget.clientHeight))) ){
				Client.drag.frameHeight = Client.drag.frameHeight - parseInt(Client.drag.dragTarget.clientHeight);
			}else{
				Client.drag.frameHeight = Client.drag.frameHeight - 200;
			}
		},
		// continue dragging
		dragDiv: function(e){
			if(!Client.drag.drag){return};
			if(!e){var e=window.event};
			// move div element
			var left=Client.drag.dragX+e.clientX-Client.drag.dragOffX;
			var top=Client.drag.dragY+e.clientY-Client.drag.dragOffY;
			//limit dragging the window to prevent going off the sides / top
			if( left < 0 ){
				left = 0;
			}else if (left > (Client.drag.frameWidth) ){
				left = Client.drag.frameWidth;
			}
			if( top < 0 ){
				top = 0;
			}else if (top > (Client.drag.frameHeight) ){
				top = Client.drag.frameHeight;
			}

			Client.drag.dragTarget.style.left=left+'px';
			Client.drag.dragTarget.style.top=top+'px';
			return false;
		},
		// stop dragging
		stopDrag:function(){
			Client.drag.drag=false;
		}
	},
	/* end dragable Div Code */

	playsound: function(){
		if(this.allowSound){
			var sound = document.getElementById("playsnd");
			try{
				sound.play();
			}catch(e){
				//we are IE or other browser that isn't html5, try IE/opera bgsound
				try{
					var sc = document.getElementById("sndcontainer");
					var src = sc.attributes.getNamedItem("src");
					src = (src !== null) ? src.value : "";
					sc.innerHTML = "<span><bgsound loop=0 src='" + src + "'></span>";
				}catch(e2){}
			}
		}
	},

	showAgentPhoto: function(strURL, strTitle){
		Client.templateShowAgentPhoto(strURL, strTitle);
	},

	/* Agent Typing Code start */
	agentsTypingAdd: function(displayName){
		if (displayName != undefined && displayName != null){
			if (this.agentsTypingNames.length == 0){
				this.agentsTypingNames.push( displayName );
			} else {
				for (var search=0;search<this.agentsTypingNames.length;search++){
					if (this.agentsTypingNames[search]===displayName){
						return false;
					}
				}
				this.agentsTypingNames.push( displayName );
			}
		}
		Client.templateAgentsTyping(this.agentsTypingNames);
		return true;
	},

	agentsTypingRemove: function(displayName){
		var returnValue = false;

		if (this.agentsTypingNames == undefined || 
			this.agentsTypingNames.length == 0 || 
			displayName == undefined || 
			displayName == null || 
			displayName == "") {
			return returnValue;
		}

		var index = String(this.agentsTypingNames).indexOf(displayName);

		if ( index != -1 ) {
			this.agentsTypingNames.splice(index,1);
			returnValue = true;
		}
		
		Client.templateAgentsTyping(this.agentsTypingNames);

		return returnValue;
	},

	agentsTypingList: function(){
		return this.agentsTypingNames;
	},

	agentsTypingRemoveAll: function(){
		this.agentsTypingNames.length = 0;
		return true;
	},
	/*Agent Typing Code end*/

	/* Branch Script code */
	bs: {
		BSScript: "",
		BSAction: "",
		BSTimer: 40,
		timerID: 0,
		inittimerid: -1,
		timerStart: function(){
			var objDiv = document.getElementById("branchscript");
			objDiv.style.display="block";
			var height = Client.getChatHeight();
			var width = Client.getChatWidth();
			Client.bs.show(true, height, width);

			Client.bs.BSTimer = 40;
			document.getElementById("bstimer").innerHTML = Client.bs.BSTimer;
			inittimerid = window.setTimeout(Client.bs.timerRun,1000);
			document.getElementById("bsok").disabled = true;
			Client.addClass(document.getElementById("bsok"),"disabled");
		},
		timerRun: function(){
		    if(Client.bs.BSTimer<=-10){
				return;
			}
		    if(Client.bs.BSTimer<=0){
		        Client.bs.cancel();
				return;
			}else{
		        Client.bs.BSTimer--;
				clearTimeout(inittimerid);
				document.getElementById("bstimer").innerHTML = Client.bs.BSTimer;
				Client.bs.timerID = window.setTimeout(Client.bs.timerRun,1000);
			}
		},
		cancel: function(){
			Client.bs.BSTimer=-10;//stop timer
			clearTimeout(Client.bs.timerID);
			clientDriver.cancelAutoPilotScript(Client.bs.BSScript);
			Client.bs.show(false);
		},
		ok: function(){
			Client.bs.BSTimer=-10;//stop timer
			clearTimeout(Client.bs.timerID);
			var sel = document.getElementById("bsselect");
		    //Modified condition to handle Mozila Firefox v45 onwards because innerText is returning Empty or "".
			var selItem = sel.options.item(sel.selectedIndex);
			var opt = (selItem.innerText) ? selItem.innerText : selItem.textContent;
			clientDriver.sendServerAutoPilotResponse(Client.bs.BSScript, sel.value, Client.bs.BSAction, opt);
			Client.bs.show(false);
		},
		setup: function(scriptid, action, question, url, resp){
			Client.bs.BSScript = scriptid;
			Client.bs.BSAction = action;

			(document.getElementsByTagName("body")[0].innerText != undefined)?
				document.getElementsByName("string92")[0].innerText = question:
				document.getElementsByName("string92")[0].textContent = question;

			if(url.length>0){
				document.getElementById("bsurl").innerHTML = "";
			}else{
				document.getElementById("bsurl").innerHTML = "<a href='"+url+"' target='"+clientDriver.PUSH_FRAME+"'>"+url+"</a>";
			}

			var sel = document.getElementById("bsselect");

			while(sel.options.length>0)
				sel.options[0]=null;

			sel.options[sel.length] = new Option(""+outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_CHOOSE_ONE), -1);

			for(var i=0;i<(resp.length-1);i+=2){
				resp[i] = resp[i].replace(/amp;/g,"");
				sel.options[sel.length] = new Option(resp[i], resp[i+1]);
			}

			document.getElementById("bsok").disabled = true;
			Client.bs.timerStart();

			if (Client.inFlyout) {
				top.postMessage('{"call":"setAlert","color":"' + g_AlertColor + '"}', '*');
			}
		},
		change: function(objSel){
			document.getElementById("bsok").disabled = (objSel.value == -1 || objSel.value == 0);
			Client.removeClass(document.getElementById("bsok"),"disabled");
		},
		show: function(visible, height, width) {
			var branchScriptElement = document.getElementById("branchscript");
			if (!branchScriptElement) {
				return;
			}
			if (visible) {
				branchScriptElement.style.display="block";
				branchScriptElement.style.left = (width - branchScriptElement.clientWidth)/ 2 + "px";
				branchScriptElement.style.top = (height - branchScriptElement.clientHeight) / 2 + "px";
			} else {
				branchScriptElement.style.display="none";
			}
		}
	},
	/* End Branch Script Code*/

	/* Customer Timeout Code */
	ct: {
		CTTimer: 30,
		cttid: -1,
		initcttid: -1,
		timeoutPrompt: function(offset){
			Client.ct.CTTimer = 30 - offset;
			document.getElementById("ctimouttimer").innerHTML = Client.ct.CTTimer;
			var height = Client.getChatHeight();
			var width = Client.getChatWidth();
			Client.ct.showTimeout(true, height, width);
			window.focus();
			Client.ct.initcttid = window.setTimeout(Client.ct.timerRun,1000);
			Client.playsound();
			if (Client.inFlyout) {
				top.postMessage('{"call":"setAlert","color":"' + g_AlertColor + '"}', '*');
			}
		},
		timerRun: function(){
			if(Client.ct.CTTimer<=-10)
				return;
			if(Client.ct.CTTimer<=0){
			    Client.ct.timeout();
				return;
			}else{
				if (Client.typing) {
					Client.ct.timeoutContinue();
					return;
				}
				Client.ct.CTTimer--;
				clearTimeout(Client.ct.initcttid);
				var timerElem = document.getElementById("ctimouttimer");
				if (timerElem) {
					timerElem.innerHTML = Client.ct.CTTimer;
				}
				Client.ct.cttid = window.setTimeout(Client.ct.timerRun,1000);
			}
		},
		timeoutContinue: function(){
		    Client.ct.CTTimer=-10;//stop timer
		    clearTimeout(Client.ct.cttid);
			clientDriver.customerTimeoutState(0);
			Client.ct.showTimeout(false);
		},
		timeoutEnd: function (exitClient) {
			if (typeof exitClient === "undefined") {
				exitClient = true;
			}
			Client.ct.CTTimer = -10;//stop timer
			clearTimeout(Client.ct.cttid);
			Client.ct.showTimeout(false);
			if (exitClient) {
				Client.exit();
			}
		},
		timeout: function(){
		    Client.ct.CTTimer=-10;//stop timer
		    clearTimeout(Client.ct.cttid);
			clientDriver.customerTimeoutState(1);
			Client.ct.showTimeout(false);
		},
		showTimeout: function(visible, height, width) {
			var timeoutElem = document.getElementById("custtimeout");
			if (!timeoutElem) {
				return;
			}
			if (visible) {
				timeoutElem.style.display="block";
				timeoutElem.style.left = (width - Client.ct.clientWidth)/ 2 + "px";
				timeoutElem.style.top = (height - Client.ct.clientHeight)/ 2 + "px";
			} else {
				timeoutElem.style.display="none";
			}
			if (this.inFlyout) {

			}
		}
	},
	/* End Customer Timeout Code */

	/* Show URL Code */
	showURL: function(url, target) {
		window.open(url, target);
	},
	/* End Show URL Code */
	/* Cobrowse Start */
	cobrowse: {
		is_cobrowse: false,
		is_cobrowse_ended: false,
		launch: function(llsessionid){

			this.is_cobrowse_ended = false;
			this.is_cobrowse = true;
			
			var router_server = g_router_server;
			var company_id = g_company_id;
			var filter_enabled = false;//TODO: putback: g_filter_enabled  -  ##COBROWSE_FILTER_ENABLED##;
			var filter_colors = g_filter_colors;
			var filter_colors_array =filter_colors.split(";");
			var fmask;
			
			var url;
			var sidurl;
			
			url = "https://";
			url = url + router_server;
			sidurl = url;

			url = url + "/" + "product/new_client.aspx?SID=";
			url = url + company_id + ":";
			url = url + llsessionid;


			sidurl = sidurl + "/" + "registersid_impl.aspx?SID=" + company_id + ":" + llsessionid;

			if (filter_enabled & 1){
				fmask = filter_colors_array[0];
				fmask = fmask.slice(1);
				url = url + "&FM1=" + fmask; 
			}

			if (filter_enabled & 2){
				fmask = filter_colors_array[1];
				fmask = fmask.slice(1);
				url = url + "&FM2=" + fmask; 
			}

			if (filter_enabled & 4){
				fmask = filter_colors_array[2];
				fmask = fmask.slice(1);
				url = url + "&FM3=" + fmask; 
			}

			if (filter_enabled & 8)
			{
				fmask = filter_colors_array[3];
				fmask = fmask.slice(1);
				url = url + "&FM4=" + fmask; 
			}

			var cim_cobrowse = "";
			cim_cobrowse = document.createElement("iframe");
			cim_cobrowse.setAttribute("id", "cobrowse_frame");
			cim_cobrowse.setAttribute("name", "cobrowse_frame");
			cim_cobrowse.setAttribute("src", sidurl);
			cim_cobrowse.style.width = "1px";
			cim_cobrowse.style.height = "1px";
			
			//cim_cobrowse.style.display = "none";    

			// *** Sid registration ***
			document.body.appendChild(cim_cobrowse);

			// *** Start CoBrowse after Sid registration ***
			setTimeout(function(){Client.cobrowse.StartCoBrowse(url)}, 5000);

			Client.templateCobrowseEnabled(true);
		},
		StartCoBrowse: function(url)
		{
			var cim_cobrowse = "";
			cim_cobrowse = document.getElementById("cobrowse_frame");
			if (cim_cobrowse)
				cim_cobrowse.setAttribute("src", url);
		},

		close: function(){
			var cim_cobrowse = "";
			var url = document.location.protocol + "//" + document.location.host;
			if (document.location.port.length > 0)
				url = url + ":" + document.location.port;
			url = url + "/netagent/";
			url = url + "blank.html";

			outputNetworkLayer.sendCobrowseEnd();
					
			Client.templateDisableButton("btnCobrowse");
			cim_cobrowse = document.getElementById("cobrowse_frame");
			cim_cobrowse.setAttribute("src", url);
			is_cobrowse_ended = true;
			
			Client.templateCobrowseEnabled(false);
		}
	},
	/* Cobrowse End */

	/* Upload Code*/
	upload: {
		attachID: -1,
		attachmentToken: -1,
		serverID: -1,
		companyID: -1,
		extensions: [],
		extListType: 1,
		WHITELIST: 1,
		BLACKLIST: 0,
		inprogress: false,
		control: null,
		telerikObject: null,
		lastProgressTime: null,
		uploadProgressUpdateTime: 10, //minimum time in seconds between progress updates to the server (actualy time depends on the progress update rate from Telerik
		attachment_url : "",
		maxFileSize : 10,
		sessionId : 0,
		enableUpload : 0,
		controlsUrl: "../includes2/radasyncmanual.aspx",
		fileName: "",
		thumbnailBase64UriData: "",
		refreshControls: true, // means upload UI needs refresh
		uploadStatus : {
			SUCCESS: 0,
			FAILED_BLACKLIST: 1,
			FAILED_WHITELIST: 2,
			UNABLE_TO_SAVE_TEMP: 3,
			UNABLE_TO_CONTACT_SERVICE: 4,
			TOKEN_BAD: 5,
			FAILED_VIRUS_SCAN: 6,
			REACHED_MAX_FILE_SIZE : 7
		},
		onUploadEnable: function(sessionId, attachmentToken, attachID, serverID, companyID, extensions, listType, fileSize, attachmentServiceUrl, enableUpload, displayMessage) {
			Client.upload.sessionId = sessionId;
			Client.upload.attachmentToken = attachmentToken;
			Client.upload.attachID = attachID;
			Client.upload.serverID = serverID;
			Client.upload.companyID = companyID;
			Client.upload.extensions = extensions.split(',');
			Client.upload.extListType = parseInt(listType) ? Client.upload.extListType : Client.upload.BLACKLIST;
			Client.upload.maxFileSize = fileSize;
			Client.upload.attachment_url = attachmentServiceUrl;
			Client.upload.enableUpload = parseInt(enableUpload);
			Client.upload.refreshControls = true;

			Client.upload.control = null;
			Client.upload.extensionMsg = (Client.upload.extListType == Client.upload.WHITELIST) ?
				outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_ALLOWED_EXTENSIONS) :
				msg = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_DISALLOWED_EXTENSIONS);

			if (attachmentToken.trim().length > 1) {
				if (Client.upload.enableUpload) {
					if (displayMessage) {
						clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_ENABLE), null, null);
					}
					Client.templateEnableButton("upload");
				} else {
					clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_DISABLE), null, null);
					Client.templateDisableButton("upload");
				}
			} else {
				Client.templateDisableButton("upload");
			}
			Client.upload.hideUploadDlg();

			if (typeof Client.updateUI == "function") {
				Client.updateUI();
			}
		},
		showUploadDlg: function(){
			Client.templateDisableButton("upload");
			if (Client.upload.attachmentToken.length > 1 && !Client.upload.inprogress) {
				var objDiv = document.getElementById("uploadwindow");
				if (objDiv) {
					objDiv.style.display="block";
					objDiv.style.left = (Util.getDocumentDimensions().width - objDiv.clientWidth)/2 + "px";
					objDiv.style.top = (Util.getDocumentDimensions().height - objDiv.clientHeight)/2 + "px";
				}
				var ext = document.getElementById("uploadextensions");
				var extbtn = document.getElementById("btnExtension");
				if (ext && extbtn) {
					ext.innerHTML = Client.upload.extensionMsg;
					extbtn.innerHTML = Client.upload.extensionMsg;
				}
				Client.upload.lastProgressTime = null;
				var frmElement = getFrameControl();
				if (frmElement) {
					frmElement.src = Client.upload.controlsUrl + "?maxFileSize=" + Client.upload.maxFileSize;
				}
			}

			if (typeof Client.updateUI == "function") {
				Client.updateUI();
			}
		},
		hideUploadDlg: function(){
			var objDiv = document.getElementById("uploadwindow");
			var objDivExt = document.getElementById("divExtension");
			if (objDivExt != null && objDivExt != null) {
				objDivExt.style.display="none";
				objDiv.style.display="none";
			}
			Client.upload.cancelUpload();
			var frmElement = getFrameControl();
			if (frmElement != null) {
				frmElement.src = "";
			}
		},
		hideUploadDlgWithMessage: function(){
		    clientDriver.displayMsg(new Date(), '0', outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_UNAUTHORIZEDACCESS), null, null);
			var objDiv = document.getElementById("uploadwindow");
			var objDivExt = document.getElementById("divExtension");
			if (objDiv && objDivExt) {
			    objDivExt.style.display="none";
			    objDiv.style.display="none";
			}
			Client.upload.cancelUpload();
			var frmElement = getFrameControl();
			if (frmElement) {
			    frmElement.src = "";
			}
		},
		uploadStart: function(){
			Client.frameDisableButton("uploadstart");
			if(Client.upload.control!=null){
				Client.upload.control.startUpload();
				Client.upload.inprogress = true;
			}
		},
		uploadClose: function(){
			clientDriver.onUploadEnded(Client.upload.attachID);
			Client.upload.hideUploadDlg();
		},
		cancelUpload: function() {
			if (Client.upload.inprogress) {
				if (Client.upload.telerikObject && Client.upload.control) {
					var cancelBtn = Client.upload.telerikObject.$(".ruCancel", Client.upload.control.get_element());
					if (cancelBtn.length) {
						cancelBtn.click();
					}
				}
			    Client.upload.inprogress = false;
				Client.upload.attachmentToken = "";
				clientDriver.onUploadEnded(Client.upload.attachID);
			}

			if (typeof Client.updateUI == "function") {
				Client.updateUI();
			}
		},
		validateFileExtension: function (fileName) {
			var ext = fileName;
			var pos = ext.lastIndexOf('.');
			if (pos > 0)
				ext = ext.substring(pos + 1).toLowerCase();

			var fileExtensionPositionInExtList = String(Client.upload.extensions).toLowerCase().indexOf(ext);
			if ((Client.upload.extListType == Client.upload.WHITELIST && fileExtensionPositionInExtList == -1) ||
                (Client.upload.extListType == Client.upload.BLACKLIST && fileExtensionPositionInExtList > -1)) {
				return false;
			}
			return true;
		},
		callbacks: {
			OnClientFileSelected: function (sender, args) {
				Client.upload.fileName = args.get_fileName();
				var pos = Client.upload.fileName.lastIndexOf("\\");
				if (pos > 0) {
					Client.upload.fileName = Client.upload.fileName.substring(pos + 1);
				}

				var fileIsValid = Client.upload.validateFileExtension(Client.upload.fileName);
				Client.upload.validFile = fileIsValid;
				Client.upload.refreshControls = !fileIsValid; // Flag for custom templates; indicates whether upload controls should be reloaded. See comment below.
				Client.upload.inprogress = fileIsValid;

				if (fileIsValid) {
					Client.frameEnableButton("uploadstart");
				} else {
					Client.templateShowUploadError(outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_NOT_ALLOWED));
					var failStatus = (Client.upload.extListType == Client.upload.WHITELIST) ? Client.upload.uploadStatus.FAILED_WHITELIST : Client.upload.uploadStatus.FAILED_BLACKLIST;
					clientDriver.sendUploadPrevented(Client.upload.attachID, failStatus, Client.upload.fileName);
					// Force reload of upload controls, because otherwise a failed file-validation causes the next successful
					// upload to send the file that failed validation.
					Client.upload.showUploadDlg();
				}

				if (typeof Client.updateUI == "function") {
					Client.updateUI();
				}
			},
			OnClientFileUploading: function(sender, args) {
				if (Client.upload.validFile) {
			        var obj = { attachID: Client.upload.attachID,
			            attachmentToken: Client.upload.attachmentToken,
			            serverID: Client.upload.serverID,
			            companyID: Client.upload.companyID,
			            sessionID: Client.upload.sessionId,
			            attachmentURL: Client.upload.attachment_url
			        };
			        args.set_queryStringParams(obj);
				
					outputNetworkLayer.OnUploadStarted(Client.upload.attachID, Client.upload.fileName);
					clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangStringWithVariable(OutputNetworkLayer.LANG_STRING_UPLOAD_STARTED, Client.upload.fileName), null, null);
				} else {
					args.set_cancel(true);
					Client.upload.cancelUpload();
				}

				if (typeof Client.updateUI == "function") {
					Client.updateUI();
				}
			},
			OnClientProgressUpdating: function(sender, args) {
				var data = args.get_data();
				var percent = data.percent;
				var now = new Date();
				
				if(Client.upload.lastUploadTime==null || percent == 100 || Client.upload.lastUploadTime.getTime() + (Client.upload.uploadProgressUpdateTime * 1000) < now.getTime() ){
					Client.upload.lastUploadTime = now;
					outputNetworkLayer.OnUploadProgress(Client.upload.attachID,percent);
				}
			},
			OnClientFileUploaded: function(sender, args) {
				Client.upload.inprogress = false;
				Client.upload.attachmentToken = "";

				outputNetworkLayer.OnUploadProgress(Client.upload.attachID,100);
				clientDriver.onUploadEnded(Client.upload.attachID);
				clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_UPLOAD_ENDED), null, null);
					if (typeof Client.updateUI == "function") {
						Client.updateUI();
				} else {
					setTimeout(function() {
						Client.upload.hideUploadDlg();
					}, 3000);
				}
			},
			OnClientFileUploadFailed: function(sender, args) {
				args.set_handled(true);

				if(Client.upload.inprogress){
					var reason = Client.upload.uploadStatus.UNABLE_TO_CONTACT_SERVICE;
			        if(args._message.indexOf("404") > 0)
			            reason = Client.upload.uploadStatus.UNABLE_TO_CONTACT_SERVICE;
			        else if(args._message.indexOf("500") > 0)
			            reason = Client.upload.uploadStatus.UNABLE_TO_SAVE_TEMP;
			        else if(args._message.indexOf("503") > 0)
			            reason = Client.upload.uploadStatus.UNABLE_TO_CONTACT_SERVICE;
				    clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_FAILED), null, null);
				    clientDriver.onUploadEnded(Client.upload.attachID);
				    outputNetworkLayer.OnUploadFailed(Client.upload.attachID,reason);
			    }
				Client.upload.inprogress = false;
				Client.upload.attachmentToken = "";

				if (typeof Client.updateUI == "function") {
					Client.updateUI();
				} else {
					setTimeout(Client.upload.hideUploadDlg,1000);
				}
			},
			OnClientValidationFailed: function(sender, args) {
				Client.upload.fileName = args.get_fileName();
				var pos = Client.upload.fileName.lastIndexOf("\\"); //repeated code from on file selected to ensure that we have the correct file name.
				if (pos > 0) {
					Client.upload.fileName = Client.upload.fileName.substring(pos + 1);
				}
				Client.upload.inprogress = false;
				Client.upload.refreshControls = true

				Client.templateShowUploadError(outputNetworkLayer.getLangStringWithVariable(OutputNetworkLayer.LANG_STRING_FILE_SIZE_EXCEEDED,sender._maxFileSize));
				clientDriver.sendUploadPrevented(Client.upload.attachID, Client.upload.uploadStatus.REACHED_MAX_FILE_SIZE, Client.upload.fileName);
				sender.deleteAllFileInputs();

				Client.frameEnableButton("uploadstart");
				if (typeof Client.updateUI == "function") {
					Client.updateUI();
				}
			},
		}
	},
	/* End Upload*/
	
	clickPoweredBy: function(){
		if (document.getElementById("poweredby"))
			window.open("http://www.gomoxie.com", "_blank");
	},
	
	getLatestJvmVer: function() {
		if (deployJava.versionCheck('1.8+'))
			return "1.8";
		else if (deployJava.versionCheck('1.7'))
			return "1.7";
		else if (deployJava.versionCheck('1.6'))
			return "1.6";
		else if (deployJava.versionCheck('1.5'))
			return "1.5";
		else if (deployJava.versionCheck('1.4'))
			return "1.4";
		else if (deployJava.versionCheck('1.1+'))
			return "1.1";
		else
			return null;
	},
	
	enableExitButton: function(enable){
		if (enable){
			Client.setupCopy();
			Client.setupPrint();
			Client.setupUpload();
		}
		Client.buttonDisable("exit",!enable);
	},
	
    // Set "splitter" point between content and footer based on footer's height
	setSplitter: function() {
	    var contentElem = document.getElementById("content");
	    var footerElem = document.getElementById("footer");
	    if (contentElem && footerElem) {
	        contentElem.style.bottom = (footerElem.offsetHeight + Client.SPLITTER_MARGIN) + "px";
	    }
	},
	
	setupCopy: function(){
		if(g_ShowCopyButton==false){
			Client.buttonHide("copy",true);
			return;
		}
		if (window.clipboardData) {
			Client.use_ie_clipboard = true;
			Client.buttonHide("copy",false);
		} else {
			if(Client.copybtn!=null && pluginlist.indexOf("Flash")!=-1){
				//setup ZeroClipboard
				Client.clip = new ZeroClipboard( Client.copybtn, {   moviePath: "../includes2/ZeroClipboard.swf" } );
				Client.clip.addEventListener( 'load', function(client) {
                    Client.buttonHide("copy",false);
                } );
				Client.clip.on( 'noflash', function ( client, args ) {
					Client.buttonHide("copy",true);
				} );
				Client.clip.on( 'wrongflash', function ( client, args ) {
					Client.buttonHide("copy",true);
				} );
				Client.clip.on( 'complete', function(client, args) {
				    var toast = new Android_Toast({content:  outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_COPIED_TO_CLIPBOARD), position: 'bottom'});
				} );
				//TODO: nationalize the above text
				window.onresize = function(){
					Client.clip.reposition();
				};
				
				if(Client.copyEleContent!=null){
					//This tells ZeroClipboard what text to copy when the user mouses over the copy button
					Client.copybtn.onmouseover = function(){
						Client.clip.setText( 
							(Client.copyEleContent.innerText != undefined) ? 
								Client.copyEleContent.innerText : Client.copyEleContent.textContent );
					};
				}
			}
		}
	},
	
	setupPrint: function(){
		if(g_ShowPrintButton==false){
			Client.buttonHide("print",true);
			return;
		}
	},
	setupUpload: function(){
		if(g_ShowAttachButton==false){
			Client.buttonHide("upload",true);
			return;
		}
	},
	copyTranscript: function(copyElm){
		if(Client.use_ie_clipboard){
			var oRange = document.body.createTextRange();
			oRange.moveToElementText(copyElm);
			oRange.execCommand("copy", false, 0);
		}else{
			//zeroClipboard - nothing to do
		}
	},

	printTranscript: function(){
		setTimeout("window.print();",100);
	},
	
	showEmailTranscript: function(){
		var objDiv = document.getElementById("emailtranscript");
		objDiv.style.display="block";
		var w=0;
		var h=0;
		if(document.clientWidth && document.clientHeight){
			w = document.clientWidth;
			h= document.clientHeight;
		}else if (document.documentElement && document.documentElement.clientWidth){
			w = document.documentElement.clientWidth;
			h = document.documentElement.clientHeight;
		}else if (document.body){
			w = document.body.clientWidth;
			h = document.body.clientHeight;
		}
		objDiv.style.left = (w - objDiv.clientWidth)/ 2 + "px";
		objDiv.style.top = (h - objDiv.clientHeight)/ 2 + "px";
	},
	
	hideEmailTranscript: function(){
		var objDiv = document.getElementById("emailtranscript");
		objDiv.style.display="none";

	},
	
	emailTranscriptSubmit: function(){
		var ele= document.getElementById("emailtranscriptaddress");
		if(ele!=null){
			var val = ele.value;
			if(val.match(/.+@.+\..+/)){
				clientDriver.sendEmailTranscriptAddress(val);
			}else{
			    var toast = new Android_Toast({content:  outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_EMAIL_TRANSCRIPT_INVALID), position: 'bottom'});
				return;
			}
		}
		this.hideEmailTranscript();
	},
	
	transcriptResult: function(code){
		// Show an error if transcript-email submit failed
		if (code != 0) {
			clientDriver.displayMsg(new Date(), "0", outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_EMAIL_TRANSCRIPT_ERROR), null, null)
		};
	},
	
	queuePositionUpdate: function(status) {
		clientDriver.displayMsg(new Date(), Client.MSG_FORMAT_SCRIPT, status, null, null);
	},
	
	/*called to display a new chat message. Just calls the template specific version
	 *
	 * @param type				The type of message (constants in ClientDriver).
	 * @param time				The time string.
	 * @param name				The name of the user sending the message.
	 * @param message			The message (with HTML) to display.
	 * @param typeDesc			A nationalized description of the message type.
	 * @param url				A url to have a link to at the end of the message.
	 * @param target			The frame that the url should open in.
	 * @param urlmsg			The text that should link to the url which should be displayed at the end of the message.
	 * @param id				An optional id that if specified will be set on the html containing the message.  Can be used later to replace the message.
	 * @param evType			An optional value that indicates the original event type recorded in the database.
	 */
	addMessage: function(type, time, name, message, typeDesc, url, target, urlmsg, id, evType) {
		Client.templateAddMessage(type, time, name, message, typeDesc, url, target, urlmsg, id, evType);
		Client.agentsTypingRemove(name);
	},
	
	/* called to replace a chat message with a different message
	* ('Translating...' with the actual translated message
	*
	*/
	replaceMessage: function(id, text){
		Client.templateReplaceMessage(id, text);
	},
	
	
	/* The following functions are intended for the Template to override
	so that they can customize functionality. Methods that show a return value
	in this example require a return value of that type in order to operate correctly.
	(and they are defaulted with a resonable implementation in case they aren't overridden). 
	
	The can be overridden something like this:
	Client.templateStartup = function(){return true;};	
	Client.templateLoadStrings = function(){};
	Client.templateConnected = function(){};
	Client.templateAddMessage = function(type, time, name, message, typeDesc, url, target, urlmsg, id){};
	Client.templateReplaceMessage = function(id, text){};
	Client.templateSetStatusMsg = function(encodedhtml){};
	Client.templateGetStatusDisplayType = function(msgid){};
	Client.templateEnableButton = function(buttonid){};
	Client.templateDisableButton = function(buttonid){};
	Client.templateHideButton = function(buttonid){};
	Client.templateShowButton = function(buttonid){};
	Client.templateEnableChat = function(enable){};
	Client.templateEnableInputArea = function(enable){};
	Client.templateShowAgentPhoto = function(strURL, strTitle){};
	Client.templateGetSessionStartMessage = function(agentName){return "Session Started"};
	Client.templateSetSessionId = function(sesid){};
	Client.templateAgentsTyping = function(agentNamesArray){};
	Client.templateTextSent() = function(){};
	Client.templateTextOnFocus() = function(event){};
    Client.templateShowUploadError = function(msg){};
    Client.templateInitFlyout() = function(){};
	
	Additionally, the template should set the value for Client.templateName, and Client.supportedFeatures.
	Client.supportedFeatures should bew the sum of the Client.FEATURE_* values that the template supports.
	*/
	
	//called upon startup of the client
	templateStartup: function(){
		return true;
		/* this specifically should be overwridden to do startup functionality and return true if everything is ok */
	},
	
	//called after a connection is made to the server. 
	templateConnected: function(){},
	
	//called after the language string are loaded
	templateLoadStrings: function(){},
	
	/* Called to display a new chat message
	 * @see templateAddMessage
	 */
	templateAddMessage: function(type, time, name, message, typeDesc, url, target, urlmsg, id, evType){},
	
	/* called to replace a chat message with a different message
	* ('Translating...' with the actual translated message)
	* @param id			the id of the message to replace
	* @param text		the text to replace the current message with
	 */
	templateReplaceMessage: function(id, text){},
	
	//called when the status message should be updated
	templateSetStatusMsg: function(encodedhtml){},
	
	//called to determine how to display certain status messages (newly updated ones) - return -1 for no display, one of MSG_FORMAT_* (probably not CUST) or 10 to display as a normal status message
	templateGetStatusDisplayType: function(msgid){
		return 10;
	},
	
	//called to enable a button, a default implementation is provided, but can be overridden
	templateEnableButton: function(buttonid){
		var button = document.getElementById(buttonid);
		if(button!=null){
			button.disabled = false;
			button.className = "";
		}
	},
	
	//called to disable a button, a default implementation is provided, but can be overridden
	templateDisableButton: function(buttonid){
		var button = document.getElementById(buttonid);
		if(button!=null){
			button.disabled = true;
			button.className = "disabled";
		}
	},
	frameDisableButton: function(buttonid){
		var button = window.frames["ifrmUpload"].document ?
			window.frames["ifrmUpload"].document.getElementById(buttonid) :
			window.frames["ifrmUpload"].contentWindow.document.getElementById(buttonid);
	    if(button!=null){
	        button.disabled = true;
	        button.className = "disabled";
	    }
	},	
	frameEnableButton: function(buttonid){
	    var button = window.frames["ifrmUpload"].document ?
			window.frames["ifrmUpload"].document.getElementById(buttonid) :
			window.frames["ifrmUpload"].contentWindow.document.getElementById(buttonid);
	    if(button!=null){
	        button.disabled = false;
	        button.className = "";
	    }
	},	
	//called to hide a button. a default implementation is provided, but can be overridden
	templateHideButton: function(buttonid){
		var button = document.getElementById(buttonid);
		if(button!=null){
			button.style.display = "none";
		}
	},
	
	//called to show a button. a default implementation is provided, but can be overridden
	templateShowButton: function(buttonid){
		var button = document.getElementById(buttonid);
		if(button!=null){
			button.style.display = "";
		}
	},
	
	//called when chat should be enabled or disabled (agent joins or leaves)
	templateEnableChat: function(enable){},
	
	//called when the input area should be enabled or disabled (agent joins or leaves)
	templateEnableInputArea: function(enable){
		setTimeout("try{Client.textArea.focus();}catch(e){}", 100);
	},
	
	//called when the agent photo should be enabled (or disabled if URL is empty or null)
	templateShowAgentPhoto: function(strURL, strTitle){},
	
	//called when cobrowse is started and ended - mostly to display/hide an end cobrowse button
	templateCobrowseEnabled: function(enable){},
	
	//called when the session starts to get the message that should be displayed.  A reasonable default is provided, but can be overridden.
	templateGetSessionStartMessage: function(agentName){
		return outputNetworkLayer.getLangStringWithVariable(OutputNetworkLayer.LANG_STRING_CONNECT_WITH, agentName);
	},
	
	//called when the session id is set. A reasonable default is provided, but can be overridden.
	templateSetSessionId: function(sesid){
		if (document.getElementById('sesid') != undefined){
			document.getElementById('sesid').value = ""+sesid;
			document.getElementById('sesid').innerHTML = ""+sesid;
		}
	},
	
	templateGetSessionId: function(){
		if (document.getElementById('sesid') != undefined){
			return document.getElementById('sesid').value ;
		}
		return 0;
	},
	
	//called when the typing agents changes, is passed an array of agent names
	templateAgentsTyping: function(agentNamesArray){},

	//called just after the user send a text message. Can but used to put a 'type here' message in the text area or other effects.
	templateTextSent: function(){},
	
	//called when the text input area gets focus.
	templateTextOnFocus: null,
	
	//called when the typed text but the customer changes length (the length is passed). Useful if you want to display remaining characters (g_MaxMsgSize is max message size).
	templateTextLength: function(len){
	    if(g_MaxMsgSize > 0){
	        var ele = document.getElementById('textcount');
	        var left = g_MaxMsgSize - len;
	        if(ele!=null){
	            if(left < 21){
	                if(ele.style.visibility == 'hidden'){
	                    ele.style.visibility = 'visible';
	                }
	                ele.innerHTML = left;
	            }else{
	                if(ele.style.visibility != 'hidden'){
	                    ele.style.visibility = 'hidden';
	                    ele.innerHTML = '';
	                }
	            }
	        }
	    }
	},

    //called when a file for upload is blocked
	templateShowUploadError: function(msg){
	    var toast = new Android_Toast({content: msg, position: 'bottom'});
	},

	//called after templateStartup if the template is in a flyout, so the template specific code can setup the flyout
	//default implementation just sets up the hide button, changes the closed tab text, and makes the chat background transparent (so flyout background shows through)
	templateInitFlyout: function() {
		Client.setFlyoutButton("MoxieFlyoutBtn1", true);

		if(!NAParam.dontConnect) {
			top.postMessage('{"call":"setFlyoutTabText","text":"' + safeHTMLEncode(g_Flyout_ReturnToChat_Text) + '"}', '*');
		}

		document.body.style.backgroundColor = "rgba(0,0,0,0)";
	},

	// Toggle visibility of flyout button (which may be owned by template if the template is not in the flyout).
	// Assuming the template is rendered in a flyout, a message is sent to the containing window with information. The message data specifies
	// the button's id, text, access-key, title, and a message to send to the inner window when the button's click event fires.
	// @param btnName: Must be "MoxieFlyoutBtn1" or "MoxieFlyoutBtn2" - other values result in nothing happening.
	// @param enabled: True means the button should be visible and available to click.
	setFlyoutButton: function(btnName, enabled) {
		var btnMessages = {
			MoxieFlyoutBtn1: {
				text: safeHTMLEncode(g_Flyout_Hide_Text),
				message: "hideChat",
				accessKey: "h",
				title: safeHTMLEncode(g_Flyout_Hide_Text)
			},
			MoxieFlyoutBtn2: {
				textId: OutputNetworkLayer.LANG_STRING_END_CHAT,
				message: "endChat",
				accessKey: (function() {
					var e = document.getElementById("end");
					return e && e.getAttribute("accesskey");
				})(),
				title: (function() {
					var e = document.getElementById("end");
					return e && e.getAttribute("title");
				})()
			}
		}

		var msg = btnMessages[btnName];
		if (msg) {
			msg.id = btnName;
			msg.call = "setFlyoutBtn";
			msg.text = msg.textId ? outputNetworkLayer.getLangString(msg.textId) : msg.text;
			if (!enabled) {
				msg.text = "";
				msg.accessKey = "";
				msg.message = "";
				msg.accessKey = "";
			}

			if (Client.inFlyout) {
				top.postMessage(JSON.stringify(msg), "*");
			}
		}
	}
};


// Telerik control callback methods
function OnClientFileSelected(sender, args) {
    Client.upload.callbacks.OnClientFileSelected(sender,args);
}
function OnClientFileUploading(sender, args) {
	Client.upload.callbacks.OnClientFileUploading(sender, args);
}
function OnClientProgressUpdating(sender, args) {
    Client.upload.callbacks.OnClientProgressUpdating(sender, args);
}
function OnClientFileUploaded(sender, args) {
	Client.upload.callbacks.OnClientFileUploaded(sender, args);
}
function OnClientFileUploadFailed(sender, args) {
	Client.upload.callbacks.OnClientFileUploadFailed(sender, args);
}
function OnClientValidationFailed(sender, args) {
    Client.upload.callbacks.OnClientValidationFailed(sender, args);
}
function OnClientAddedHandler(upload, eventArgs) {
	Client.upload.control = upload;

	var frameWindow = window.frames["ifrmUpload"].contentWindow ?
		window.frames["ifrmUpload"].contentWindow :
		window.frames["ifrmUpload"];

	Client.upload.telerikObject = frameWindow.getTelerikObject();

	var docRoot = frameWindow.document;

	var i;
	if (upload.get_localization()) {
		upload.get_localization().Select = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_SELECT);
		upload.get_localization().Cancel = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_CANCEL);
		upload.get_localization().Remove = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_FILE_UPLOAD_REMOVE);
	}
	
	for (i=0;i<250;i++) {
		var element = docRoot.getElementsByName("string"+i);
		if(element != undefined && element != null && element.length>0){
			var text = ""+outputNetworkLayer.getLangString(i);
			var count;
			for(count=0;count<element.length;count++){
				if(element[count]!= undefined && element[count] != null){
					if( element[count].tagName.toUpperCase() == "INPUT" )
						element[count].value = safeHTMLEncode(text);
					else
						element[count].innerHTML = safeHTMLEncode(text);
				}
			}
		}
	}

	// Add title attribute to file-input
	var fileInput = docRoot.querySelector(".ruFileInput");
	if (fileInput) {
		fileInput.title = outputNetworkLayer.getLangString(OutputNetworkLayer.LANG_STRING_UPLOAD_MESSAGE);
	}
	}

function listDownExtensions() {
	var table = document.getElementById('tblExtension');
	if(table.rows.length > 2)
		return;
	var extList = Client.upload.extensions;
	var extCount = extList.length;
	var rowCount = table.rows.length;
	var row = 0;
	var index = 0;
	while(true)
	{
		var row1 = table.insertRow(rowCount-1+row);
		for(var col = 0; col < 6 && extCount > 0; col++,index++)
		{
			var cell1 = row1.insertCell(col);
			if(extCount > index)
				cell1.innerHTML = extList[index];
			else
				cell1.innerHTML = "&nbsp;";
			cell1.style.width = "16%";
		}
		row++;
		if(extCount <= index)
			return;
	}
}

function setExtensionDialogSize()
{
    var l_height = screen.width;
    if (document.getElementById("divExtension") && l_height > 500) 
    {
        document.getElementById("divExtension").style.width = "420px";
    }
    else
    {
        document.getElementById("divExtension").style.width = parseInt(l_height - 25) + "px";
    }
    var table = document.getElementById('tblExtension');
    var size = table.rows.length * 25;
    if(size < 125)
        document.getElementById("divExtension").style.height = parseInt(size)+"px";
    else
        document.getElementById("divExtension").style.height = "125px";
}	
